def match_by_key(tuple_to_search, dictionary_to_match, key_to_match_on):
    matched = None

    for item in tuple_to_search:
        if getattr(item, key_to_match_on) == getattr(dictionary_to_match, key_to_match_on):
            matched = item
            break

    return matched



import logging
import re
from looker_sdk import models
from looker_deployer.utils import deploy_logging
from looker_deployer.utils.get_client import get_client
from looker_deployer.utils.match_by_key import match_by_key

logger = deploy_logging.get_logger(__name__)


def get_filtered_roles(source_sdk, pattern=None):
    roles = source_sdk.all_roles()
    roles = [i for i in roles if not i.name == "Admin"]
    logger.debug(
        "Roles pulled",
        extra={
            "roles_names": [i.name for i in roles]
        }
    )

    if pattern:
        compiled_pattern = re.compile(pattern)
        roles = [i for i in roles if compiled_pattern.search(i.name)]
        logger.debug(
            "Roles filtered",
            extra={
                "filtered_roles": [i.name for i in roles],
                "pattern": pattern
            }
        )

    return roles


def write_roles(roles, target_sdk, pattern=None, allow_delete=None):

    # INFO: Get all roles from target instances that match pattern for name
    target_roles = get_filtered_roles(target_sdk, pattern)

    # INFO: Get all permission sets and models from target instances
    target_permission_sets = target_sdk.all_permission_sets()
    target_model_sets = target_sdk.all_model_sets()

    # INFO: Start Loop of Create/Update on Target
    for role in roles:
        # INFO: Create Role
        new_role = models.WriteRole()
        new_role.__dict__.update(role.__dict__)

        # INFO: For the role being created or updated, need to swap the
        # permission set and model set ids
        matched_permission_set = match_by_key(target_permission_sets,
                                              role.permission_set, "name")
        matched_model_set = match_by_key(target_model_sets,
                                         role.model_set, "name")
        new_role.permission_set_id = matched_permission_set.id
        new_role.model_set_id = matched_model_set.id

        # INFO: Test if role is already in target
        matched_role = match_by_key(target_roles, role, "name")

        if matched_role:
            role_exists = True
        else:
            role_exists = False

        # INFO: Create or Update the role
        if not role_exists:
            logger.debug("No Role found. Creating...")
            logger.debug("Deploying Role", extra={"role": role.name})
            matched_role = target_sdk.create_role(new_role)
            logger.info("Deployment complete", extra={"role": new_role.name})
        else:
            logger.debug("Existing Role found. Updating...")
            logger.debug("Deploying Role", extra={"role": new_role.name})
            matched_role = target_sdk.update_role(matched_role.id, new_role)
            logger.info("Deployment complete", extra={"role": new_role.name})

    # INFO: Delete missing roles that are not in the source
    if allow_delete:
        for target_role in target_roles:

            # INFO: Test if model set is already in target
            matched_role = match_by_key(roles, target_role, "name")

            if not matched_role:
                logger.debug("No Source Role found. Deleting...")
                logger.debug("Deleting Role", extra={"role": target_role.name})
                target_sdk.delete_role(target_role.id)
                logger.info("Delete complete",
                            extra={"role": target_role.name})


def send_roles(source_sdk, target_sdk, pattern=None, allow_delete=None):
    # INFO: Get all roles from source instance
    roles = get_filtered_roles(source_sdk, pattern)
    write_roles(roles, target_sdk, pattern, allow_delete)


def main(args):

    if args.debug:
        logger.setLevel(logging.DEBUG)

    source_sdk = get_client(args.ini, args.source)

    for t in args.target:
        target_sdk = get_client(args.ini, t)
        send_roles(source_sdk, target_sdk, args.pattern, args.delete)



import logging
import re
from looker_sdk import models
from looker_deployer.utils import deploy_logging
from looker_deployer.utils.get_client import get_client
from looker_deployer.utils.match_by_key import match_by_key

logger = deploy_logging.get_logger(__name__)


def get_filtered_permission_sets(source_sdk, pattern=None):
    permission_sets = source_sdk.all_permission_sets()

    logger.debug(
        "Permission Sets pulled",
        extra={
            "permission_sets_names": [i.name for i in permission_sets]
        }
    )

    permission_sets = [i for i in permission_sets if not i.built_in]

    if pattern:
        compiled_pattern = re.compile(pattern)
        permission_sets = [i for i in permission_sets
                           if compiled_pattern.search(i.name)]
        logger.debug(
            "Permission Sets filtered",
            extra={
                "filtered_permission_sets": [i.name for i in permission_sets],
                "pattern": pattern
            }
        )

    return permission_sets


def write_permission_sets(permission_sets, target_sdk, pattern=None,
                          allow_delete=None):

    # INFO: Get all permission sets from target instances that match pattern
    target_permission_sets = get_filtered_permission_sets(target_sdk, pattern)

    # INFO: Start Loop of Create/Update on Target
    for permission_set in permission_sets:
        # INFO: Create permission set
        new_permission_set = models.WritePermissionSet()
        new_permission_set.__dict__.update(permission_set.__dict__)

        # INFO: Test if permission set is already in target
        matched_permission_set = match_by_key(target_permission_sets,
                                              permission_set, "name")

        if matched_permission_set:
            permission_set_exists = True
        else:
            permission_set_exists = False

        # INFO: Create or Update the permission set
        if not permission_set_exists:
            logger.debug("No Permission Set found. Creating...")
            logger.debug("Deploying Permission Set",
                         extra={"permission_set": permission_set.name})
            matched_permission_set = target_sdk.create_permission_set(
                new_permission_set)
            logger.info("Deployment complete",
                        extra={"permission_set": new_permission_set.name})
        else:
            logger.debug("Existing permission set found. Updating...")
            logger.debug("Deploying Permission Set",
                         extra={"permission_set": new_permission_set.name})
            matched_permission_set = target_sdk.update_permission_set(
                matched_permission_set.id,
                new_permission_set)
            logger.info("Deployment complete",
                        extra={"permission_set": new_permission_set.name})

    # INFO:    Delete missing permission sets that are not in source
    if allow_delete:
        for target_permission_set in target_permission_sets:

            # INFO:    Test if model set is already in target
            matched_permission_set = match_by_key(permission_sets,
                                                  target_permission_set,
                                                  "name")

            if not matched_permission_set:
                logger.debug("No Source Permission Set found. Deleting...")
                logger.debug("Deleting Permission Set",
                             extra={"permission_set":
                                    target_permission_set.name})
                target_sdk.delete_permission_set(target_permission_set.id)
                logger.info("Delete complete",
                            extra={"permission_set":
                                   target_permission_set.name})


def send_permission_sets(source_sdk, target_sdk,
                         pattern=None, allow_delete=None):
    # INFO:    Get all permissions sets from source instance
    permission_sets = get_filtered_permission_sets(source_sdk, pattern)
    write_permission_sets(permission_sets, target_sdk, pattern, allow_delete)


def main(args):

    if args.debug:
        logger.setLevel(logging.DEBUG)

    source_sdk = get_client(args.ini, args.source)

    for t in args.target:
        target_sdk = get_client(args.ini, t)
        send_permission_sets(source_sdk, target_sdk, args.pattern, args.delete)



import logging
import re
from looker_sdk import models
from looker_deployer.utils import deploy_logging
from looker_deployer.utils.get_client import get_client
from looker_deployer.utils.match_by_key import match_by_key

logger = deploy_logging.get_logger(__name__)


def get_filtered_groups(source_sdk, pattern=None):
    groups = source_sdk.all_groups()

    logger.debug(
        "Groups pulled",
        extra={
            "groups_names": [i.name for i in groups]
        }
    )

    groups = [i for i in groups if not i.externally_managed]

    if pattern:
        compiled_pattern = re.compile(pattern)
        groups = [i for i in groups if compiled_pattern.search(i.name)]
        logger.debug(
            "Groups filtered",
            extra={
                "filtered_groups": [i.name for i in groups],
                "pattern": pattern
            }
        )

    return groups


def write_groups(groups, target_sdk, pattern=None, allow_delete=None):

    # INFO: Get all groups from target instances that match pattern for name
    target_groups = get_filtered_groups(target_sdk, pattern)

    # INFO: Start Loop of Create/Update on Target
    for group in groups:
        # INFO: Create Group
        new_group = models.WriteGroup()
        new_group.__dict__.update(group.__dict__)

        # INFO: Test if group is already in target
        matched_group = match_by_key(target_groups, group, "name")

        if matched_group:
            group_exists = True
        else:
            group_exists = False

        # INFO: Create or Update the Group
        if not group_exists:
            logger.debug("No Group found. Creating...")
            logger.debug("Deploying Group", extra={"group": group.name})
            matched_group = target_sdk.create_group(new_group)
            logger.info("Deployment complete", extra={"group": new_group.name})
        else:
            logger.debug("Existing Group found. Updating...")
            logger.debug("Deploying Group", extra={"group": new_group.name})
            matched_group = target_sdk.update_group(matched_group.id,
                                                    new_group)
            logger.info("Deployment complete", extra={"group": new_group.name})

    # INFO: Delete missing groups that are not in the source
    if allow_delete:
        for target_group in target_groups:

            # INFO: Test if model set is already in target
            matched_group = match_by_key(groups, target_group, "name")

            if not matched_group:
                logger.debug("No Source Group found. Deleting...")
                logger.debug("Deleting Group", extra={"group":
                             target_group.name})
                target_sdk.delete_group(target_group.id)
                logger.info("Delete complete", extra={"group":
                            target_group.name})


def send_groups(source_sdk, target_sdk, pattern=None, allow_delete=None):
    # INFO: Get all groups from source instance
    groups = get_filtered_groups(source_sdk, pattern)
    write_groups(groups, target_sdk, pattern, allow_delete)


def main(args):
    if args.debug:
        logger.setLevel(logging.DEBUG)

    source_sdk = get_client(args.ini, args.source)

    for t in args.target:
        target_sdk = get_client(args.ini, t)
        send_groups(source_sdk, target_sdk, args.pattern, args.delete)



import logging
import re
from looker_sdk import models
from looker_deployer.utils import deploy_logging
from looker_deployer.utils.get_client import get_client
from looker_deployer.utils.match_by_key import match_by_key

logger = deploy_logging.get_logger(__name__)


def get_filtered_groups(source_sdk, pattern=None):
    groups = source_sdk.all_groups()

    logger.debug(
        "Groups pulled",
        extra={
            "groups_name": [i.name for i in groups]
        }
    )

    groups = [i for i in groups if not i.externally_managed]

    if pattern:
        compiled_pattern = re.compile(pattern)
        groups = [i for i in groups if compiled_pattern.search(i.name)]
        logger.debug(
            "Groups in Group filtered",
            extra={
                "filtered_groups": [i.name for i in groups],
                "pattern": pattern
            }
        )

    return groups


def write_groups_in_group(source_sdk, target_sdk, pattern=None):  # noqa: C901

    # INFO: Get all groups from source and target instances that match pattern
    # for name
    groups = get_filtered_groups(source_sdk, pattern)
    target_groups = get_filtered_groups(target_sdk, pattern=None)

    # INFO: Start Loop of Create/Update on Target
    for group in groups:
        matched_group = match_by_key(target_groups, group, "name")
        logger.debug("Group Matched " + matched_group.name)
        groups_in_group = source_sdk.all_group_groups(group.id)
        target_groups_in_group = target_sdk.all_group_groups(matched_group.id)

        # INFO: Need to loop through the groups in group to identify
        # target group ID
        for i, nested_group in enumerate(groups_in_group):
            target_nested_group = match_by_key(target_groups, nested_group,
                                               "name")

            if target_nested_group:
                nested_group.id = target_nested_group.id
                groups_in_group[i] = nested_group
            else:
                groups_in_group.remove(nested_group)

        # INFO: If groups in groups between instances is different, we need to
        # either delete or create
        source_group_ids = []
        for nested_group in groups_in_group:
            source_group_ids.append(nested_group.id)

        target_group_ids = []
        for nested_group in target_groups_in_group:
            target_group_ids.append(nested_group.id)

        all_group_ids = list(set().union(source_group_ids, target_group_ids))

        for group_id in all_group_ids:
            in_source = True
            in_target = True
            try:
                source_group_ids.index(group_id)
            except Exception:
                in_source = False

            try:
                target_group_ids.index(group_id)
            except Exception:
                in_target = False

            if in_source and not in_target:
                logger.debug("No Groups in Group found. Creating...")
                logger.debug("Deploying Groups in Group",
                             extra={"group_name": group.name,
                                    "group_group_id": group_id})
                target_sdk.add_group_group(group_id=matched_group.id,
                                           body=models.GroupIdForGroupInclusion
                                           (group_id=group_id))
                logger.info("Deployment Complete",
                            extra={"group_name": group.name,
                                   "group_group_id": group_id})

            elif not in_source and in_target:
                logger.debug("Extra Groups in Group found. Deleting...")
                logger.debug("Removing Groups in Group",
                             extra={"group_name": group.name,
                                    "group_group_id": group_id})
                target_sdk.delete_group_from_group(group_id=matched_group.id,
                                                   deleting_group_id=group_id)
                logger.info("Deployment Complete",
                            extra={"group_name": group.name,
                                   "group_group_id": group_id})


def main(args):
    if args.debug:
        logger.setLevel(logging.DEBUG)

    source_sdk = get_client(args.ini, args.source)

    for t in args.target:
        target_sdk = get_client(args.ini, t)
        write_groups_in_group(source_sdk, target_sdk, args.pattern)



import logging
import re
from looker_sdk import models
from looker_deployer.utils import deploy_logging
from looker_deployer.utils.get_client import get_client
from looker_deployer.utils.match_by_key import match_by_key

logger = deploy_logging.get_logger(__name__)


def get_filtered_model_sets(source_sdk, pattern=None):
    model_sets = source_sdk.all_model_sets()

    logger.debug(
        "Model Sets pulled",
        extra={
            "model_sets_names": [i.name for i in model_sets]
        }
    )

    model_sets = [i for i in model_sets if not i.built_in]

    if pattern:
        compiled_pattern = re.compile(pattern)
        model_sets = [i for i in model_sets if compiled_pattern.search(i.name)]
        logger.debug(
            "Model Sets filtered",
            extra={
                "filtered_model_sets": [i.name for i in model_sets],
                "pattern": pattern
            }
        )

    return model_sets


def write_model_sets(model_sets, target_sdk, pattern=None, allow_delete=None):

    # INFO: Get filtered model sets from target Instance
    target_model_sets = get_filtered_model_sets(target_sdk, pattern)

    # INFO: Start Loop of Create/Update on Target
    for model_set in model_sets:
        # INFO: Create model set
        new_model_set = models.WriteModelSet()
        new_model_set.__dict__.update(model_set.__dict__)

        # INFO: Test if model set is already in target
        matched_model_set = match_by_key(target_model_sets, model_set, "name")

        if matched_model_set:
            model_set_exists = True
        else:
            model_set_exists = False

        # INFO: Create or Update the Model Set
        if not model_set_exists:
            logger.debug("No Model Set found. Creating...")
            logger.debug("Deploying Model Set",
                         extra={"model_set": model_set.name})
            matched_model_set = target_sdk.create_model_set(new_model_set)
            logger.info("Deployment complete",
                        extra={"model_set": new_model_set.name})
        else:
            logger.debug("Existing model set found. Updating...")
            logger.debug("Deploying Model Set",
                         extra={"model_set": new_model_set.name})
            matched_model_set = target_sdk.update_model_set(
                matched_model_set.id, new_model_set)
            logger.info("Deployment complete",
                        extra={"model_set": new_model_set.name})

    # INFO: Delete missing model sets that are not in source
    if allow_delete:
        for target_model_set in target_model_sets:

            # INFO: Test if model set is already in target
            matched_model_set = match_by_key(model_sets, target_model_set,
                                             "name")

            if not matched_model_set:
                logger.debug("No Source Model Set found. Deleting...")
                logger.debug("Deleting Model Set",
                             extra={"model_set": target_model_set.name})
                target_sdk.delete_model_set(target_model_set.id)
                logger.info("Delete complete",
                            extra={"model_set": target_model_set.name})


def send_model_sets(source_sdk, target_sdk, pattern=None, allow_delete=None):
    # INFO: Get all model sets from source instance
    model_sets = get_filtered_model_sets(source_sdk, pattern)
    write_model_sets(model_sets, target_sdk, pattern, allow_delete)


def main(args):

    if args.debug:
        logger.setLevel(logging.DEBUG)

    source_sdk = get_client(args.ini, args.source)

    for t in args.target:
        target_sdk = get_client(args.ini, t)
        send_model_sets(source_sdk, target_sdk, args.pattern, args.delete)



import logging
import re
from looker_sdk import models
from looker_deployer.utils import deploy_logging
from looker_deployer.utils.get_client import get_client
from looker_deployer.utils.match_by_key import match_by_key

logger = deploy_logging.get_logger(__name__)


def get_filtered_user_attributes(source_sdk, pattern=None):
    user_attributes = source_sdk.all_user_attributes()

    logger.debug(
        "User Attributes pulled",
        extra={
            "user_attribute_names": [i.name for i in user_attributes]
        }
    )

    user_attributes = [i for i in user_attributes if not i.is_system]

    if pattern:
        compiled_pattern = re.compile(pattern)
        user_attributes = [i for i in user_attributes
                           if compiled_pattern.search(i.name)]

    logger.debug(
        "User Attributes filtered",
        extra={
            "filtered_user_attributes": [i.name for i in user_attributes],
            "pattern": pattern
        }
    )

    return user_attributes


def get_user_attribute_group_value(source_sdk, user_attribute):
    user_attribute_group_value = \
        source_sdk.all_user_attribute_group_values(user_attribute.id)

    logger.debug("User Attribute Group Value Pulled", extra={
        "group_ids": [i.group_id for i in user_attribute_group_value]
    })

    return user_attribute_group_value


def match_user_attributes(source_user_attribute, target_user_attributes):
    matched_user_attribute = None

    for target_user_attribute in target_user_attributes:
        if source_user_attribute.name == target_user_attribute.name:
            matched_user_attribute = target_user_attribute

    return matched_user_attribute


def add_group_name_information(source_sdk, list_to_update):
    for i, item in enumerate(list_to_update):
        item_group_name = source_sdk.group(group_id=item.group_id)
        item.name = item_group_name.name
        list_to_update[i] = item
    return list_to_update


def write_user_attributes(source_sdk, target_sdk,
                          pattern=None, allow_delete=None):

    # INFO: Get All User Attirbutes From Source Instance
    user_attributes = get_filtered_user_attributes(source_sdk, pattern)
    target_user_attributes = get_filtered_user_attributes(target_sdk, pattern)
    target_groups = target_sdk.all_groups()

    # INFO: Start Loop of Create/Update User Attribute on Target
    # and Update Group Values if set
    for user_attribute in user_attributes:
        # INFO: Create user attribute
        new_user_attribute = models.WriteUserAttribute()
        new_user_attribute.__dict__.update(user_attribute.__dict__)

        # INFO: Test if user attribute is already in target
        matched_user_attribute = match_user_attributes(user_attribute,
                                                       target_user_attributes)
        if matched_user_attribute:
            user_attribute_exists = True
        else:
            user_attribute_exists = False

        # INFO: Create or Update the User Attribute
        if not user_attribute_exists:
            logger.debug("No User Attribute found. Creating...")
            logger.debug("Deploying User Attribute",
                         extra={"user_attribute": new_user_attribute.name})
            matched_user_attribute = target_sdk.create_user_attribute(new_user_attribute)
            logger.info("Deployment complete",
                        extra={"user_attribute": new_user_attribute.name})
        else:
            logger.debug("Existing user attribute found. Updating...")
            logger.debug("Deploying User Attribute",
                         extra={"user_attribute": new_user_attribute.name})
            matched_user_attribute = target_sdk.update_user_attribute(
                matched_user_attribute.id,
                new_user_attribute)
            logger.info("Deployment complete",
                        extra={"user_attribute": new_user_attribute.name})
        # INFO: Set group values for user attribute
        user_attribute_group_values = get_user_attribute_group_value(
            source_sdk, user_attribute)
        user_attribute_group_values = add_group_name_information(
            source_sdk, user_attribute_group_values)
        # INFO: Need to loop through the group values in a user attribute to
        # determine matching group
        for i, user_attribute_group_value in enumerate(
                user_attribute_group_values):
            target_group = match_by_key(
                target_groups, user_attribute_group_value, "name")
            if target_group:
                user_attribute_group_value.group_id = target_group.id
                user_attribute_group_values[i] = user_attribute_group_value
            else:
                user_attribute_group_values.remove(user_attribute_group_value)

        if user_attribute_group_values:
            target_sdk.set_user_attribute_group_values(
                user_attribute_id=matched_user_attribute.id,
                body=user_attribute_group_values)

    # INFO: Delete missing users attirbutes that are not in source
    if allow_delete:
        for target_user_attribute in target_user_attributes:

            # INFO: Test if user attribute is already in target
            matched_user_attribute = match_by_key(
                user_attributes, target_user_attribute, "name")

            if not matched_user_attribute:
                logger.debug("No Source User Attribute found. Deleting...")
                logger.debug("Deleting User Attribute",
                             extra={"user_attribute":
                                    target_user_attribute.name})
                target_sdk.delete_user_attribute(target_user_attribute.id)
                logger.info("Delete complete",
                            extra={"user_attribute":
                                   target_user_attribute.name})


def main(args):

    if args.debug:
        logger.setLevel(logging.DEBUG)

    source_sdk = get_client(args.ini, args.source)

    for t in args.target:
        target_sdk = get_client(args.ini, t)
        write_user_attributes(source_sdk, target_sdk,
                              args.pattern, args.delete)



# Copyright 2021 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from looker_deployer import version
from looker_sdk import api_settings, methods, requests_transport, auth_session, serialize, _settings
from typing import Optional


def configure_sdk(
    config_file: str = "looker.ini",
    section: Optional[str] = None,
    config_settings: Optional[api_settings.ApiSettings] = None,
) -> methods.Looker31SDK:
    """Default dependency configuration"""
    settings = (
        _settings(config_file, section) if config_settings is None else config_settings
    )
    settings.is_configured()
    settings.headers['User-Agent'] = f"Looker Deployer {version.__version__}"
    transport = requests_transport.RequestsTransport.configure(settings)
    return methods.Looker31SDK(
        auth_session.AuthSession(settings, transport, serialize.deserialize31, "3.1"),
        serialize.deserialize31,
        serialize.serialize31,
        transport,
        "3.1",
    )


def get_client(ini, env):
    sdk = configure_sdk(config_file=ini, section=env)
    return sdk



import logging
import re
from looker_deployer.utils import deploy_logging
from looker_deployer.utils.get_client import get_client
from looker_deployer.utils.match_by_key import match_by_key

logger = deploy_logging.get_logger(__name__)


def get_filtered_roles(source_sdk, pattern=None):
    roles = source_sdk.all_roles()

    logger.debug(
        "Roles pulled",
        extra={
            "role_names": [i.name for i in roles]
        }
    )

    if pattern:
        compiled_pattern = re.compile(pattern)
        roles = [i for i in roles if compiled_pattern.search(i.name)]
        logger.debug(
            "Rolesfiltered",
            extra={
                "filtered_roles": [i.name for i in roles],
                "pattern": pattern
            }
        )

    return roles


def write_role_to_group(source_sdk, target_sdk, pattern=None):

    # INFO: Get all roles and groups information
    roles = get_filtered_roles(source_sdk, pattern)
    target_roles = get_filtered_roles(target_sdk, pattern)
    target_groups = target_sdk.all_groups()

    # INFO: Start Loop of Update on Target
    for role in roles:
        matched_role = match_by_key(target_roles, role, "name")
        role_groups = source_sdk.role_groups(role.id)

        # INFO: Need to loop through the role groups and find
        # corresponding target group match
        for i, role_group in enumerate(role_groups):
            target_group = match_by_key(target_groups, role_group, "name")

            if target_group:
                role_group.id = target_group.id
                role_groups[i] = role_group
            else:
                role_groups.remove(role_group)

        # INFO: Perform update to target role group
        groups_for_update = [i.id for i in role_groups]
        logger.debug("Updating Role Group. Updating...")
        logger.debug("Deploying Role Group",
                     extra={"role_name": role.name,
                            "group_ids": groups_for_update})
        target_sdk.set_role_groups(role_id=matched_role.id,
                                   body=groups_for_update)
        logger.info("Deployment Complete",
                    extra={"role_name": role.name,
                           "group_ids": groups_for_update})


def main(args):
    if args.debug:
        logger.setLevel(logging.DEBUG)

    source_sdk = get_client(args.ini, args.source)

    for t in args.target:
        target_sdk = get_client(args.ini, t)
        write_role_to_group(source_sdk, target_sdk, args.pattern)
